Game = class 'Game'
Game.TimePerFrame = 1 / 30

function Game:__init()
	self._stateStack = StateStack()
--	self._statsFrames = 0
--	self._statsSeconds = 0
	
	self._stateStack:registerState('Game', require 'gamestate')
	self._stateStack:pushState('Game')
end

function Game:_handleEvent()
	levent.pump()
	
	for name, a, b, c, d, e, f in levent.poll() do
		local event = { type = name }
		
		if name == 'directorydropped' then
			event.path = a
		elseif name == 'filedropped' then
			event.path = a
		elseif name == 'focus' then
			event.focus = a
		elseif name == 'keypressed' then
			event.key = a
			event.scancode = b
			event.isrepeat = c
		elseif name == 'keyreleased' then
			event.key = a
			event.scancode = b
		elseif name == 'mousefocus' then
			event.focus = a
		elseif name == 'mousemoved' then
			event.x, event.y = a, b
			event.dx, event.dy = c, d
		elseif name == 'mousepressed'
		    or name == 'mousereleased' then
			event.x, event.y = a, b
			event.button = c
		elseif name == 'resize' then
			event.width, event.height = a, b
		elseif name == 'textedited' then
			event.text = a
			event.start = b
			event.length = c
		elseif name == 'textinput' then
			event.text = a
		elseif name == 'visible' then
			event.visible = a
		elseif name == 'wheelmoved' then
			event.x, event.y = a, b
		end
		
		self._stateStack:handleEvent(event)
		
		if name == 'quit' then
			lwindow.close()
		end
	end
end

function Game:_update(dt)
	self._stateStack:update(dt)
end

--[=[
function Game:_updateStats(dt)
	self._statsSecond = self._statsSecond + dt
	self._statsFrames = self._statsFrames + 1
	
	if self._statsSecond >= 1 then
		self._statsSecond = self._statsSecond - 1;
		self._statsFrames = 0;
	end
end
]=]

function Game:_draw()
	if lgraphics.isActive() then
		lgraphics.clear(lgraphics.getBackgroundColor())
		lgraphics.origin()
		self._stateStack:draw()
		lgraphics.present()
	end
end

function Game:run()
	lmath.setRandomSeed(os.time())
	
	local currentTime, previousTime = ltimer.getTime(), ltimer.getTime()
	local timeSinceLastUpdate = 0
	
	while lwindow.isOpen() do
		local deltaTime = currentTime - previousTime
		previousTime = currentTime
		currentTime = ltimer.getTime()
		timeSinceLastUpdate = timeSinceLastUpdate + deltaTime
		
		print(deltaTime)
		
		while timeSinceLastUpdate > self.TimePerFrame do
			timeSinceLastUpdate = timeSinceLastUpdate - self.TimePerFrame
			
			self:_handleEvent()
			self:_update(self.TimePerFrame)
			
			if self._stateStack:isEmpty() then
				lwindow.close()
			end
		end
		
		self:_draw()
	end
end