StateStack = class 'StateStack'

function StateStack:__init()
	self._stack = {}
	self._state = {}
	self._pending = {}
end

function StateStack:_createState(name)
	local found = self._state[name]
	
	assert(found, 'state \'' .. name .. '\' not found')
	
	return found(self)
end

function StateStack:_applyPendingChanges()
	for _, change in ipairs(self._pending) do
		local action, name = unpack(change)
		
		if action == 'push' then
			self._stack[#self._stack + 1] = self:_createState(name)
		elseif action == 'pop' then
			self._stack[#self._stack] = nil
		elseif action == 'clear' then
			for i = 1, #self._stack do
				self._stack[i] = nil
			end
		end
	end
	
	for i = 1, #self._pending do
		self._pending[i] = nil
	end
end

function StateStack:update(dt)
	for i = #self._stack, 1, -1 do
		local state = self._stack[i]
		
		if (state.update) then
			state:update(dt)
		end
	end
	
	self:_applyPendingChanges()
end

function StateStack:draw()
	for i = 1, #self._stack do
		local state = self._stack[i]
		
		if (state.draw) then
			state:draw()
		end
	end
end

function StateStack:handleEvent(event)
	for i = #self._stack, 1, -1 do
		local state = self._stack[i]
		
		if (state.handleEvent) then
			state:handleEvent(event)
		end
	end
end

function StateStack:pushState(name)
	self._pending[#self._pending + 1] = {'push', name}
end

function StateStack:popState()
	self._pending[#self._pending + 1] = {'pop'}
end

function StateStack:clearStates()
	self._pending[#self._pending + 1] = {'clear'}
end

function StateStack:isEmpty()
	return #self._stack == 0
end

function StateStack:registerState(name, state)
	self._state[name] = state
end