local modules = {
	"audio", "event", "filesystem", "font", "graphics", "image",
	"joystick", "keyboard", "math", "mouse", "physics", "sound",
	"system", "thread", "timer", "touch", "video", "window"
}

for _ = 1, #modules do
	local module = modules[_]
	_G["l" .. module] = love[module]
end

local types = {
	{ "BezierCurve", "math" },
	{ "Body", "physics" },
	{ "Canvas", "graphics" },
	{ "ChainShape", "physics" },
	{ "Channel", "thread" },
	{ "CircleShape", "physics" },
	{ "CompressedData", "math", "compress" },
	{ "CompressedImageData", "image" },
--	{ "Contact" },
	{ "Cursor", "mouse" },
	{ "Decoder", "sound" },
	{ "DistanceJoint", "physics" },
--	{ "Drawable" }, > superclass
	{ "EdgeShape", "physics" },
	{ "File", "filesystem" },
	{ "FileData", "filesystem" },
	{ "Fixture", "physics" },
	{ "Font", "graphics" },
	{ "ImageFont", "graphics" }, -- > Font
	{ "FrictionJoint", "physics" },
	{ "GearJoint", "physics" },
	{ "GlyphData", "font" },
	{ "Image", "graphics" },
	{ "ImageData", "image" },
	{ "Screenshot", "graphics" }, -- > ImageData
--	{ "Joint" }, > superclass 
--	{ "Joystic" },
	{ "Mesh", "graphics" },
	{ "MotorJoint", "physics" },
	{ "MouseJoint", "physics" },
--	{ "Object" }, > superclass
	{ "ParticleSystem", "graphics" },
	{ "PolygonShape", "physics" },
	{ "RectangleShape", "physics" }, -- > PolygonShape
	{ "PrismaticJoint", "physics" },
	{ "PulleyJoint", "physics" },
	{ "Quad", "graphics" },
	{ "RandomGenerator", "math" },
	{ "Rasterizer", "font" },
	{ "BMFontRasterizer", "font" }, -- > Rasterizer?
	{ "TrueTypeRasterizer", "font" }, -- > Rasterizer?
	{ "RevoluteJoint", "physics" },
	{ "RopeJoint", "physics" },
	{ "Shader", "graphics" },
--	{ "Shape" }, > superclass
	{ "SoundData", "sound" },
	{ "Source", "audio" },
	{ "SpriteBatch", "graphics" },
	{ "Text", "graphics" },
--	{ "Texture" }, > superclass
	{ "Thread", "thread" },
--	{ "Variant" }, > flat table, a boolean, a string, a number or LOVE userdata.
	{ "Video", "graphics" },
	{ "VideoStream", "video" }, 
	{ "WeldJoint", "physics" },
	{ "WheelJoint", "physics" },
	{ "World", "physics" }
}

for _, type in ipairs(types) do
	local name, module, func = unpack(type)
	module = "l" .. module
	
	assert(_G[name] == nil, "global var '" .. name .. "' already exists")
	
	local new = _G[module][func and func or "new" .. name]
	_G[name] = function(...)
		return new(...)
	end
end